local window = {}

function window:new(pos, size)
	local o = {}
	
	o.pos = pos
	o.size = size
	o.rect= Rect:new(pos.x,pos.y, size.x,size.y)
	o.lit = false
	
	self.__index = self
	return setmetatable(o,self)
end


function window:draw()
	if not self.lit then return end
	
	
	love.graphics.setColor(255,255,200)
	love.graphics.rectangle("fill",self.rect)
end

function window:light()
	
	self.lit = true
	
end

local neon = {}

function neon:new(pos, image)
	local o ={}
	
	o.pos = pos
	o.image = image
	o.lit = false
	self.__index = self
	return setmetatable(o, self)
end

function neon:light()
	
	self.lit = true
	
end

function neon:draw()
	
	if not self.lit then return end
	
	love.graphics.setColor(255,255,255)
	love.graphics.draw(self.image, self.pos.x, self.pos.y)
	
end

local dot = {}

function dot:new(pos)
	local o = {}
	
	o.pos = pos
	
	self.__index = self
	return setmetatable(o, self)
end

function dot:draw()
	
	love.graphics.setColor(255,0,0)
	love.graphics.point(self.pos.x, self.pos.y)
	
	love.graphics.setFont(home.debugFont)
	love.graphics.printf(tostring(self.pos), self.pos.x, self.pos.y, math.abs(home.width - self.pos.x))
	
end

local blimp = {}



function blimp:new(start, stop, speed)
	local o = {}
	
	o.pos = start
	o.start = start
	o.stop = stop
	
	o.color = {love.math.random(90,110),love.math.random(170,190),love.math.random(170,190)}
	
	o.speed = speed or 1
	o.t=0
	
	o.dir = 1
	if o.start.x > o.stop.x then
		o.dir = -1
	end
		
	self.__index = self
	return setmetatable(o,self)
end

function blimp:update(dt)
	
	if randomPercent(1/1000) then
		self.color = {love.math.random(90,110),love.math.random(170,190),love.math.random(170,190)}
	end
	
	self.pos = self.pos + Vector2.Right * dt * self.dir * self.speed
	
	if self.dir == 1 then
		if self.pos.x > self.stop.x then
			home.blimps:remove(self)
		end
	else
		if self.pos.x < self.stop.x then
			home.blimps:remove(self)
		end
	end
	
end

function blimp:draw()
	local r,g,b,a = love.graphics.getColor()
	
	
	
	love.graphics.draw(blimpImage, self.pos.x, self.pos.y, 0, self.dir, 1)
	
	--[[ DEBUG
	love.graphics.setColor(255,0,0)
	love.graphics.point(self.pos.x, self.pos.y)
	]]

	love.graphics.setColor(self.color)
	
	local w = 40
	local h=10
	
	local offX = 30
	
	
	
	local x = self.pos.x + offX
	local y = self.pos.y + 17

	if self.dir == -1 then
		
		x = self.pos.x - (w+offX)

		
	end
	
	love.graphics.rectangle("fill", x, y, w ,h)
	
	love.graphics.setColor(r,g,b,a)
end

home = {}

function home.load()
	
	
	SetConst("music", love.audio.newSource("sound/Ripper.mp3"))
	home.gui = List:new()
	home.background = love.graphics.newImage("img/Home_Background.png")
	home.sky = love.graphics.newImage("img/Home_Layers/Sky.png")
	home.stars = love.graphics.newImage("img/Home_Layers/Stars.png")
	home.buildings_back = love.graphics.newImage("img/Home_Layers/Buildings_Back.png")
	home.buildings_front = love.graphics.newImage("img/Home_Layers/Buildings_Front.png")
	home.welcome = love.graphics.newImage("img/Home_Layers/Welcome to Withmore.png") 
	
	
	temp = function()
		m = GetConst("music")
		if m:isPlaying() then
			m:pause()
		else
			m:resume()
		end
	end
	
	pauseGraphics = {
		["default"] = love.graphics.newImage("img/buttonStyle2Default.png"),
		["over"] = love.graphics.newImage("img/buttonStyle2Over.png"),
		["down"] = love.graphics.newImage("img/buttonStyle2Down.png"),
	}
	
	SetConst("pause", Button:new(Vector2:new(10,10) , temp, pauseGraphics))
	
	home.front_windows = List:new()
	
	windowWidth = 30
	windowSize = Vector2:new(windowWidth, windowWidth)
	base = Vector2:new(170, 250)
	
	windowLayout = {
	{1, 1, 1, 0, 0, 0, 0, 0, 0, 0,},
	{1, 1, 1, 0, 0, 0, 0, 0, 1, 1,},
	{1, 1, 0, 0, 0, 0, 0, 0, 0, 0,},
	{0, 1, 0, 0, 0, 0, 0, 0, 1, 0,},
	{1, 1, 0, 1, 1, 0, 1, 1, 0, 0,},
	{1, 1, 0, 0, 0, 0, 1, 0, 0, 1,},
	{1, 1, 0, 1, 0, 1, 0, 0, 0, 1,},
	{0, 1, 0, 0, 0, 0, 1, 0, 0, 0,},
	{1, 0, 1, 1, 1, 1, 0, 0, 1, 1,},
	{0, 1, 0, 0, 0, 0, 0, 0, 0, 0,},
	}
	
	offset = windowWidth + 20
	
	for y, row in ipairs(windowLayout) do
		
		for x, value in ipairs(row) do
			
			if value == 1 then 
				newPos = base + Vector2:new((x-1)*offset, (y-1)*offset)
				
				home.front_windows:add(window:new(newPos, windowSize))
			end
		end
		
	end
	
	
	neon1=neon:new(Vector2:new(-70, 720), love.graphics.newImage("img/Neon1.png"))
	neon2=neon:new(Vector2:new(610, 700), love.graphics.newImage("img/Neon2.png"))
	home.front_windows:add(neon1)
	home.front_windows:add(neon2)
	
	
	blimpImage = love.graphics.newImage("img/blimp.png")
	home.blimps = List:new()
	
	home.blimpLocs = 
	{
		{Vector2:new(100, 170),Vector2:new(668, 130)}, --Change 130 to 115 if Vert movement is too dsitracting
		{Vector2:new(90, 219),Vector2:new(-100, 219)}, --Fine, Blimp hidden
		{Vector2:new(63, 212),Vector2:new(327, 180)}, --Fine, blimp hidden
		{Vector2:new(340, 212),Vector2:new(668, 212)},
	}
	
	
	
	home.debugFont = love.graphics.newFont(11)
	
	home.t = 0
	home.length = 300
	
	local apart = 250
	local height = 300
	local width = 1.5
	
	love.window.setMode(700,850)

	home.width = love.window.getWidth()
	
	local b1 = Vector2:new((love.window.getWidth()/2)-((0.5*apart)+(100*width)), height)
	local b2 = Vector2:new((love.window.getWidth()/2)+(0.5*apart), height)
	
	temp = function() EndState("disc") end
	home.gui:add(Button:new(b1, temp, GetConst("buttonStyle1"), TextObject:new(Vector2:new(), "Discography", GetConst("buttonFont")), Vector2:new(width, 1)))
	
	temp = function() EndState("info") end
	home.gui:add(Button:new(b2, temp, GetConst("buttonStyle1"), TextObject:new(Vector2:new(), "Band info", GetConst("buttonFont")), Vector2:new(width, 1)))
	
	temp = function() EndState("quit") end
	home.gui:add(Button:new(Vector2:new(500, 750) ,temp, GetConst("buttonStyle1"), TextObject:new(Vector2:new(), "QUIT", GetConst("buttonFont"))))
	
	temp = function() EndState("merch") end
	home.gui:add(Button:new(Vector2:new((home.width/2)-width*50, height), temp, GetConst("buttonStyle1"), TextObject:new(Vector2:new(), "Shop", GetConst("buttonFont")), Vector2:new(width, 1)))
	
	home.gui:add(GetConst("pause"))
	
	--[[
	Icon = love.image.newImageData("img/icon.png")
	love.window.setIcon(Icon)
	]]--
end

function home.onStart(args)
	
	love.window.setMode(700,850,{["borderless"]=true})
	love.window.setTitle("Home")
	
	local m = GetConst("music")
	
	if not m:isPlaying() and not m:isPaused() then
		
		m:play()
		
	end
end

function home.mousereleased(x,y,button)
	
	home.gui:mousereleased(x,y,button)
	
end

function home.mousepressed(x,y,button)
	
	home.gui:mousepressed(x,y,button)
	
	
	print(tostring(Vector2:new(x,y)))
	--home.gui:add(dot:new(Vector2:new(x,y))) --DEBUG
	
end

function home.keypressed(key)
	
	if key == " " then
		
		home.blimps:add(blimp:new(home.blimpLocs[2][1], home.blimpLocs[2][2], 5) )
		
	end
	
end

function home.update(dt)
	
	
	
	home.gui:mousehover(love.mouse.getX(), love.mouse.getY())
	home.blimps:update(dt)
	
	if home.t < home.length then
		home.t = home.t + dt
		home.percent = home.t/home.length
	end
	
	if home.t > home.length then home.t = home.length end
	
	windowsToLight = home.front_windows:get(randomPercentGet, (home.percent/2000))
	
	for _, val in ipairs(windowsToLight.items) do
		
		val:light()
		
	end
	
	GetConst("music"):setVolume(remap(home.t, 0, 5, 0, 0.4, true))
	
	spawnBlimp = randomPercent(1/3000)

	if spawnBlimp then
		print("Spawning a blimp!")
		
		repeat 
		
			locs = randomSelect(home.blimpLocs)
		
		until locs ~= lastPicked
		
		lastPicked = locs
		
		home.blimps:add(blimp:new(locs[1], locs[2], love.math.random(5,15)))
		
	end
	
end

function home.draw()
	
	
	skyCol = remap(home.t, 0, home.length, 255, 150, true)
	love.graphics.setColor(skyCol, skyCol, skyCol)
	love.graphics.draw(home.sky)
	
	starAlpha = remap(home.t, 0, home.length, 0, 200, true)
	love.graphics.setColor(255,255,255,starAlpha)
	love.graphics.draw(home.stars)
	
	backCol = remap(home.t, 0, home.length, 255, 80, true)
	love.graphics.setColor(backCol, backCol, backCol)
	home.blimps:draw()

	love.graphics.draw(home.buildings_back)
	
	
	love.graphics.setColor(255,255,255)
	love.graphics.draw(home.buildings_front)

	home.front_windows:draw()
	
	
	
	
	welcomeAlpha = remap(home.t, 0, 20, 255, 0, true)
	
	love.graphics.setColor(255,255,255, welcomeAlpha)
	
	love.graphics.draw(home.welcome)
	
	
	love.graphics.setColor(255,255,255)
	home.gui:draw()
	
	--[[
	love.graphics.setFont(home.debugFont)
	love.graphics.print((home.t).." out of "..home.length.." seconds.",50,50)
	]]--
end

RegState("home",home)
RegDefaultState("home")