--LIBRARIES Start

	libs = {"Tserial","AnAL","utils","list", "vector", "timer","graphics","button"}


	for _, v in ipairs(libs) do
		print("Importing library \""..v.."\"")
		require(v)
	end

--LIBRARIES End

--STATES Start
	modules = 
	{
	"enter",
	"home",
	"info",
	"disc",
	"merch",
	"quit",
	}

	STATE = ""

	DEFAULT_STATE = ""
	INITIAL_STATE = ""

	states = {}

	function RegState(state, mod)
		print("New state registered: "..state)
		states[state]=mod
	end

	function RegDefaultState(state)
		DEFAULT_STATE = state
	end

	function RegInitialState(state)
		INITIAL_STATE = state
	end

	function EndState(state, args)
		if not state then
			state = DEFAULT_STATE
		end
		
		STATE = state
		
		if states[STATE].onStart then
			states[STATE].onStart(args)
		end
	end


	--require modules here

	for _, v in ipairs(modules) do
		require(v)
	end

--STATES End

--CONSTANTS Start
Constants = {}

	function SetConst(name, val)
		Constants[name] = val
	end

	function GetConst(name)
		return Constants[name]
	end

--CONSTANTS End

--TIMERS Start
timers = {}

function RegTimer(length, f, args, loop)
	
	print("Registered Timer in state "..STATE)
	
	if(timers[STATE]) then
		timers[STATE]:add(Timer:new(length, f, args, loop))
	else
		timers[STATE] = List:new({Timer:new(length, f, args, loop)})
	end
	

end

function DelTimer(t, s)
	
	timers[s]:remove(t)
	
end

--TIMERS End

function love.load()
		
	--Set up Constants like the Player, and other graphical entities
	
	EndSound = love.audio.newSource("sound/end.wav","static")
	quitting = false
	
	print("Entering loading stage")
	for _, v in ipairs(modules) do
		STATE = v
		print("Loading "..v)
		
		if states[v].load then
			states[v].load()
		end
		
	end
	print("Done loading")
	
	STATE = INITIAL_STATE
	states[STATE].onStart()
	
end

function love.update(dt)
	
	--main updating
	if(timers[STATE]) then
		timers[STATE]:update(dt)
	end
	
	

	if states[STATE].update then
		states[STATE].update(dt)
	end
	
	if quitting and not EndSound:isPlaying() then
		
		love.event.quit()
		
	end
	
end

function love.draw()
	
	if states[STATE].draw then
		states[STATE].draw()
	end
	
end

function love.keypressed(key)
	
	if key == "escape" and STATE ~= INITIAL_STATE then
		
		EndState()
		return
	end
	
	if states[STATE].keypressed then
		print("Entering keypressed of "..STATE)
		states[STATE].keypressed(key)
	end
end

function love.mousepressed(x,y,button)
	
	if states[STATE].mousepressed then
		states[STATE].mousepressed(x,y,button)
	end
	
end

function love.mousereleased(x,y,button)
	
	if states[STATE].mousereleased then
		states[STATE].mousereleased(x,y,button)
	end
	
end

